#! /usr/bin/env python3

import asyncio
import os
import signal
import logging

from websockets import serve

from . import stream, model, protocol

# variables
WS_PORT = 8081

terminate = {}

def shutdown_server( _, client, _2 ):
    """Command to kill the server"""
    if client.role != model.ROLE_STAFF:
        logging.warning("%s tried to use allocate, but is not staff", client.id)
        return

    terminate['object'].set_result(True)

protocol_handlers = {
    'allocate': protocol.handle_allocate,
    'notify': protocol.handle_notify,
    'notifyGroup': protocol.handle_notify_group,
    'kick': protocol.handle_kick,
    'forceRename': protocol.handle_rename,
    'shutdown': shutdown_server
}

def kill_server(*_):
    """Terminate the server"""
    loop = asyncio.get_running_loop()
    loop.run_in_executor( None, lambda: terminate['object'].set_result(True))
signal.signal(signal.SIGINT, kill_server)

async def main():
    """Main server loop"""
    socket_server = stream.SocketServer(protocol_handlers)

    async with serve( socket_server.handler, "0.0.0.0", WS_PORT ):
        terminate['object'] = asyncio.Future()
        await terminate['object']

asyncio.run( main() )
